<?php
class testCalculatesExpectedNoccMetricForClassWithDirectChildren
{

}

class c1 extends testCalculatesExpectedNoccMetricForClassWithDirectChildren {}
class c2 extends testCalculatesExpectedNoccMetricForClassWithDirectChildren {}
class c3 extends testCalculatesExpectedNoccMetricForClassWithDirectChildren {}