<?php
class PHP_Depend_Parser
{
    /**
     * Method comment block.
     *
     * @param string  $foo    A foo string.
     * @param Bar     $bar    A bar instance.
     * @param integer $foobar A foobar integer.
     * 
     * @return void
     */
    public function parse($foo, Bar &$bar = null, $foobar = 23) {
    }
}