<?php
class testCalculateDITMetricForInternalParentIncrementsMetricWithTwo extends testCalculateDITMetricForInternalParentIncrementsMetricWithTwoParent
{

}

class testCalculateDITMetricForInternalParentIncrementsMetricWithTwoParent extends ArrayObject
{
    
}