<?php
class testNPathComplexityForConsecutiveIfElseIfStatements
{
    function testNPathComplexityForConsecutiveIfElseIfStatements()
    {
        if (true) {
        } else if (true) {
        } else if (true) {
        } else {
        }
    }
}