<?php
class testParentReferenceHasExpectedStartLine extends testParentReferenceHasExpectedStartLineParent {
    public function testParentReferenceHasExpectedStartLine()
    {
        return new parent();
    }
}