<?php
class testNPathComplexityForConsecutiveIfElsifStatements
{
    function testNPathComplexityForConsecutiveIfElsifStatements()
    {
        if (true) {
        } elseif (true) {
        } elseif (true) {
        } else {
        }
    }
}