<?php
class testUnserializedClassStillReferencesSameParentInterface
    implements testUnserializedClassStillReferencesSameParentInterface_interface
{

}

interface testUnserializedClassStillReferencesSameParentInterface_interface
{
    
}