<?php
class testParentReferenceHasExpectedEndColumn extends testParentReferenceHasExpectedEndColumnParent {
    public function testParentReferenceHasExpectedEndColumn()
    {
        return new parent();
    }
}