<?php
class testParentReferenceHasExpectedEndLine extends testParentReferenceHasExpectedEndLineParent {
    public function testParentReferenceHasExpectedEndLine()
    {
        return new parent();
    }
}