<?php
/**
 * @package package
 */

/**
 * @package package
 */
class clazz
{
    public function m1() {}
    protected function m2() {}
    private $_p1;
}

/**
 * @package package
 */
interface interfs
{
    function m3();
    function m4();
}

function func() {}
?>
