<?php
function foo()
{
    if (true) {
        if (false) {
            if (true || false) {
                for ($i = 0; $i < 10; ++$i) {
                    
                }
            }
        }
    }
}