<?php
class testPropertyPostfixGraphForObjectPropertyArrayIndexExpression
{
    public function testPropertyPostfixGraphForObjectPropertyArrayIndexExpression()
    {
        $this->arguments[42] = func_get_args();
    }
}