<?php
/**
 * Function comment block.
 *
 * @param string  $foo    A foo string.
 * @param Bar     $bar    A bar instance.
 * @param integer $foobar A foobar integer.
 * 
 * @return void
 */
function pdepend($foo, Bar &$bar = null, $foobar = 23) {
}