<?php
/**
 * A simple class comment.
 */
class MyPropertyClass2
{
    /**
     * Comment...
     * 
     * @var MyPropertyClass2[]
     */
    private $property1;

    /**
     * Comment...
     *
     * @var array(string=>MyPropertyClass2)
     */
    public $property2;

    /**
     * Comment...
     *
     * @var array( integer => MyPropertyClass2)
     */
    protected $property3;

    /**
     * Comment...
     * 
     * @var MyPropertyClass2
     */
    protected $property4;
    
    /**
     * Comment...
     * 
     * @var integer
     */
    private $property5;
    
    /**
     * comment...
     * 
     * @var array(string => integer)
     */
    public $property6;
}
