<?php
class testTraitUseStatementWithColumnAliasHasExpectedEndLine
{
    use testTraitUseStatementWithColumnAliasHasExpectedEndLineMyTraitOne {
        testTraitUseStatementWithColumnAliasHasExpectedEndLineMyTraitOne::foo as bar; }
}
