<?php
class testAllocationExpressionGraphForParentKeyword
    extends testAllocationExpressionGraphForParentKeywordParent
{
    function testAllocationExpressionGraphForParentKeyword()
    {
        new parent;
    }
}