<?php
namespace A {
    interface I1 {}
    interface I2 {}
    interface I3 {}
}

namespace B {
    interface I1 {}
    interface I2 {}
    interface I3 {}
}

namespace C {
    interface I1 {}
    interface I2 {}
    interface I3 {}
}