<?php
/**
 * Comment...
 *
 * @return void
 * @throws RuntimeException Comment...
 */
function func1() {}

/**
 * Comment...
 *
 * @return SplObjectStore Comment...
 * @throws OutOfRangeException Test Exception comment.
 * @throws InvalidArgumentException Comment...
 */
function func2($x) {}

/**
 * Comment...
 *
 * @return SplObjectStore Comment...
 */
function func3() {}