<?php
class testGetNodeMetricsReturnsExpectedCaWithExceptionReference
{
    /**
     * @return void
     * @throws InvalidArgumentException
     * @throws OutOfBoundsException
     */
    public function foo()
    {

    }
}

class testGetNodeMetricsReturnsExpectedCaWithExceptionReference_exception1
{
    /**
     * @return void
     * @throws testGetNodeMetricsReturnsExpectedCaWithExceptionReference
     * @throws OutOfBoundsException
     */
    public function foo()
    {

    }
}

class testGetNodeMetricsReturnsExpectedCaWithExceptionReference_exception2
{
    /**
     * @return void
     * @throws testGetNodeMetricsReturnsExpectedCaWithExceptionReference
     * @throws OutOfBoundsException
     */
    public function foo()
    {

    }
}