<?php
/**
 * @return void
 * @throws testGetNodeMetricsReturnsExpectedCaForFunctionException
 */
function foo()
{

}

class testGetNodeMetricsReturnsExpectedCaForFunctionException extends Exception
{
    
}