<?php
class testGetConstantsReturnsExpectedParentConstants
    extends testGetConstantsReturnsExpectedParentConstants_parent
{

}

class testGetConstantsReturnsExpectedParentConstants_parent
{
    const FOO = 42;
    const BAR = 23;
}