<?php
class A extends B {
    private $a;
    protected $b;
    public $c;
    private function a() {}
    protected function b() {}
    public function c() {}
}
class B {
    public $a;
    protected $d;
    private $e;
    public $f;
}
?>