<?php
class testCalculateNpmMetricForClassWithAllVisibilityMethods
{
    private function foo()
    {

    }

    protected function bar()
    {

    }

    public function baz()
    {
        
    }
}
