<?php
class testCalculateIMPLMetricContainsUnknownIndirectImplementedInterface extends testCalculateIMPLMetricContainsUnknownIndirectImplementedInterfaceParent {}
class testCalculateIMPLMetricContainsUnknownIndirectImplementedInterfaceParent implements FooBar {}