<?php
class testAnalyzerGetProjectMetricsReturnsArrayWithExpectedKeys_parent
{

}

class testAnalyzerGetProjectMetricsReturnsArrayWithExpectedKeys extends testAnalyzerGetProjectMetricsReturnsArrayWithExpectedKeys_parent
{
    
}
