<?php
use FooLibrary\Bar\Baz\{ ClassA, ClassB, ClassC, ClassD as Fizbo };

use function foo\math\{ sin, cos, cosh };

use const foo\math\{ PI, E, GAMMA, GOLDEN_RATIO };

use foo\math\{ Math, const PI, function sin, function cos, function cosh };

use my\math\{ const SINDELFINGEN };

use my\ { ClassE, ClassF, ClassG, ClassH as Bofiz };

class ExtendingClass extends ClassB
{

}

class AliasExtendingClass extends Fizbo
{

}
