<?php
namespace nspace;

class Clazz {

	protected function method(array $convolutedFiles) {
		foreach ($fieldPaths as $fieldPath) {
			if (count($fieldPath) === 1) {
				$fileInformation = $convolutedFiles[$fieldPath{0}];
			}
		}
		return $untangledFiles;
	}
}
