<?php
class testGetNodeMetricsReturnsExpectedCaWithObjectInstantiation
{

}

class testGetNodeMetricsReturnsExpectedCaWithObjectInstantiation_instantiate
{
    function testGetNodeMetricsReturnsExpectedCaWithObjectInstantiation_instantiate()
    {
        return new testGetNodeMetricsReturnsExpectedCaWithObjectInstantiation();
    }
}