<?php
/**
 * This file is part of PDepend.
 *
 * PHP Version 5
 *
 * Copyright (c) 2008-2017 Manuel Pichler <mapi@pdepend.org>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   * Neither the name of Manuel Pichler nor the names of his
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @copyright 2008-2017 Manuel Pichler. All rights reserved.
 * @license http://www.opensource.org/licenses/bsd-license.php BSD License
 */

namespace PDepend\DependencyInjection;

use Symfony\Component\Config\Definition\Builder\ArrayNodeDefinition;
use Symfony\Component\Config\Definition\Builder\NodeDefinition;
use Symfony\Component\Config\Definition\Builder\TreeBuilder as BaseTreeBuilder;

/**
 * This is the class that embed the Symfony TreeBuilder and its root node.
 */
class TreeBuilder
{
    /**
     * @var ArrayNodeDefinition|NodeDefinition
     */
    protected $rootNode;

    /**
     * @var BaseTreeBuilder
     */
    protected $treeBuilder;

    /**
     * TreeBuilder constructor.
     *
     * @param string $name
     */
    public function __construct($name = 'pdepend')
    {
        $this->treeBuilder = new BaseTreeBuilder($name);

        if (method_exists($this->treeBuilder, 'getRootNode')) {
            $this->rootNode = $this->treeBuilder->getRootNode();

            return;
        }

        if (method_exists($this->treeBuilder, 'root')) {
            // Symfony < 4.2
            $this->rootNode = $this->treeBuilder->root($name);

            return;
        }
    }

    /**
     * Get the root node of the built tree.
     *
     * @return ArrayNodeDefinition|NodeDefinition
     */
    public function getRootNode()
    {
        return $this->rootNode;
    }

    /**
     * Get the base Symfony tree builder.
     *
     * @return BaseTreeBuilder
     */
    public function getTreeBuilder()
    {
        return $this->treeBuilder;
    }
}
