<?php

class A
{
    public function trait() {
        return 'trait()';
    }

    public function callable() {
        return 'callable()';
    }

    public function insteadof() {
        return 'insteadof()';
    }
}
