<?php
/**
 * @package org.pdepend
 */
class testGetParentReturnsNullWhenParentIsFiltered
    extends testGetParentReturnsNullWhenParentIsFilteredParent
{

}

/**
 * @package org.pdepend.filter
 */
class testGetParentReturnsNullWhenParentIsFilteredParent {}
