<?php
/**
 * This is the function docblock for foo
 *
 */
function foo() {
    // Function Comment 1
    $a = 1;

    // Function Comment 2
    if ( $a == 1 ) {
        echo 1;
    }

    // Function Comment 3
    echo $a;
}

