<?php
class testParserHandlesMaxNestingLevel
{
    public function doMaxNestingLevel()
    {
        $this->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel()
            ->doMaxNestingLevel();

       return $this;
    }
}