<?php
class testParentReferenceInClassWithoutParentThrowsException {
    public function testParentReferenceInClassWithoutParentThrowsException() {
        parent::testParentReferenceInClassWithoutParentThrowsException();
    }
}