<?php
class testFormalParameterWithParentTypeHint extends testFormalParameterWithParentTypeHint_parent
{
    public function setParent(parent $p)
    {

    }
}

class testFormalParameterWithParentTypeHint_parent
{
    
}