<?php
class MyClass00 { const callable = 42; }
class MyClass02 { const trait = 42; }
class MyClass03 { const extends = 42; }
class MyClass04 { const implements = 42; }
class MyClass05 { const static = 42; }
class MyClass06 { const abstract = 42; }
class MyClass07 { const final = 42; }
class MyClass08 { const public = 42; }
class MyClass09 { const protected = 42; }
class MyClass010 { const private = 42; }
class MyClass011 { const const = 42; }
class MyClass012 { const enddeclare = 42; }
class MyClass013 { const endfor = 42; }
class MyClass014 { const endforeach = 42; }
class MyClass015 { const endif = 42; }
class MyClass016 { const endwhile = 42; }
class MyClass017 { const and = 42; }
class MyClass018 { const global = 42; }
class MyClass019 { const goto = 42; }
class MyClass020 { const instanceof = 42; }
class MyClass021 { const insteadof = 42; }
class MyClass022 { const interface = 42; }
class MyClass023 { const namespace = 42; }
class MyClass024 { const new = 42; }
class MyClass025 { const or = 42; }
class MyClass026 { const xor = 42; }
class MyClass027 { const try = 42; }
class MyClass028 { const use = 42; }
class MyClass029 { const var = 42; }
class MyClass030 { const exit = 42; }
class MyClass031 { const list = 42; }
class MyClass032 { const clone = 42; }
class MyClass033 { const include = 42; }
class MyClass034 { const include_once = 42; }
class MyClass035 { const throw = 42; }
class MyClass036 { const array = 42; }
class MyClass037 { const print = 42; }
class MyClass038 { const echo = 42; }
class MyClass039 { const require = 42; }
class MyClass040 { const require_once = 42; }
class MyClass041 { const return = 42; }
class MyClass042 { const else = 42; }
class MyClass043 { const elseif = 42; }
class MyClass044 { const default = 42; }
class MyClass045 { const break = 42; }
class MyClass046 { const continue = 42; }
class MyClass047 { const switch = 42; }
class MyClass048 { const yield = 42; }
class MyClass049 { const function = 42; }
class MyClass050 { const if = 42; }
class MyClass051 { const endswitch = 42; }
class MyClass052 { const finally = 42; }
class MyClass053 { const for = 42; }
class MyClass054 { const foreach = 42; }
class MyClass055 { const declare = 42; }
class MyClass056 { const case = 42; }
class MyClass057 { const do = 42; }
class MyClass058 { const while = 42; }
class MyClass059 { const as = 42; }
class MyClass060 { const catch = 42; }
class MyClass061 { const die = 42; }
class MyClass062 { const self = 42; }
class MyClass063 { const parent = 42; }
