<?php
class testMultiLineCommentHasExpectedEndColumn
{
    /*
     * Just a multiline comment somewhere in this class.
     *
     * TO BE DONE: Comment this propery...
     */
    private static $_instance;
}