<?php
class testGetNodeMetricsReturnsExpectedCaWithPropertyReference
{
    /**
     * @var SplObjectStorage
     */
    protected $foo = null;
}


class testGetNodeMetricsReturnsExpectedCaWithPropertyReference_property
{
    /**
     * @var testGetNodeMetricsReturnsExpectedCaWithPropertyReference
     */
    protected $foo = null;
}