<?php
/**
 * A simple class comment.
 */
class MyMethodClass3
{
    /**
     * Comment...
     * 
     * @throws RuntimeException
     */
    public function __construct()
    {
    }

    /**
     * Comment...
     *
     * @return SplObjectStore Test return comment.
     * @throws OutOfRangeException Test Exception comment.
     * @throws OutOfBoundsException Test Exception comment.
     */
    private function method1()
    {
    }

    /**
     * Comment.
     *
     * @return SplSubject
     */
    private function method2()
    {
    }
}