<?php
/**
 * @package org.pdepend.tests
 */
interface testUnserializedInterfaceIsReturnedByMethodAsReturnClass
{
    /**
     * @return testUnserializedInterfaceIsReturnedByMethodAsReturnClass
     */
    function fooBarBaz23();
}