<?php
class User {
    public $mixed;
    public int $id;
    public float $money;
    public bool $active;
    public string $name;
    public array $list;
    public self $parent;
    public callable $event;
    public \Closure $fqn;
    public iterable $actions;
    public object $bag;
    public Role $role;
    public ?int $idN;
    public ?float $moneyN;
    public ?bool $activeN;
    public ?string $nameN;
    public ?array $listN;
    public ?self $parentN;
    public ?callable $eventN;
    public ?\Closure $fqnN;
    public ?iterable $actionsN;
    public ?object $bagN;
    public ?Role $roleN;
}