<?php
class testFieldDeclarationNotContainsClassReferenceWithAnnotationsDisabled
{
    /**
     * @var testFieldDeclarationNotContainsClassReferenceWithAnnotationsDisabled
     */
    protected $foo;
}