<?php
class testGetParentClassReturnsExpectedClassInstance
    extends testGetParentClassReturnsExpectedClassInstance_parent
{

}

class testGetParentClassReturnsExpectedClassInstance_parent
{
    
}
