<?php
class testGetNodeMetricsReturnsExpectedCaForFunctions
{

}

function foo(testGetNodeMetricsReturnsExpectedCaForFunctions $o)
{

}

function bar()
{
    return new testGetNodeMetricsReturnsExpectedCaForFunctions();
}

/**
 * @return testGetNodeMetricsReturnsExpectedCaForFunctions
 */
function baz()
{

}