<?php
class testUnserializedClassIsReturnedByMethodAsReturnClass
{
    /**
     * @return testUnserializedClassIsReturnedByMethodAsReturnClass
     */
    public static function getInstance()
    {

    }
}