<?php
class testGetNodeMetricsReturnsExpectedCboWithExceptionReference
{
    /**
     * @return void
     * @throws InvalidArgumentException
     * @throws OutOfBoundsException
     */
    public function foo()
    {

    }
}