<?php
class testParentReferenceHasExpectedStartColumn extends testParentReferenceHasExpectedStartColumnParent {
    public function testParentReferenceHasExpectedStartColumn()
    {
        return new parent();
    }
}