<?php
interface testUnserializedInterfaceStillReferencesSameParentInterface
    extends testUnserializedInterfaceStillReferencesSameParentInterface_parent
{

}

interface testUnserializedInterfaceStillReferencesSameParentInterface_parent
{
    
}