<?php
class testUnserializedClassStillReferencesSameParentClass
    extends testUnserializedClassStillReferencesSameParentClass_parent
{

}

class testUnserializedClassStillReferencesSameParentClass_parent
{
    
}