<?php
class A extends B {
    private function a2() {}
    public function b2() {}
    public function c2() {}
}

class B {
    public $c;
    private function a3() {}
    protected function b3() {}
    public function c3() {}
}
?>
