<?php
class testCalculateDITMetricForUnknownParentIncrementsMetricWithTwo extends testCalculateDITMetricForUnknownParentIncrementsMetricWithTwoParent
{

}

class testCalculateDITMetricForUnknownParentIncrementsMetricWithTwoParent extends FooBar {
    
}