<?php
class testParserHandlesDoubleQuoteStringAsConstantDefaultValue
{
    private $_foo = "
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
                                                                               x
";
}