<?php
class testParserReconfiguresXdebugMaxNestingLevel
{
    public function xdebugMaxNestingLevel()
    {
        $this->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel()
            ->xdebugMaxNestingLevel();

        return $this;
    }
}