<?php
/**
 * A simple class comment.
 */
class MyConstantClass
{
    /**
     * This is one comment.
     */
    const constant_with_comment = 1;

    // Wrong comment type
    const constant_without_doc_comment = 2;

    const constant_without_comment = 3;

    /**
     * This is a second comment.
     */
    const another_constant_with_comment = 4;

    public function foo()
    {
        return "42";
    }
}